//
//  ViewController.swift
//  Sample
//
//

import UIKit

class ViewController: UIViewController,RcpCommonDelegate, RcpRFIDDelegate, HWEventDelegate, UITableViewDelegate, UITableViewDataSource {


    @IBOutlet weak var tagTableView: UITableView!
    @IBOutlet weak var switchOn: UISwitch!
    @IBOutlet weak var stateLabel: UILabel!
    @IBOutlet weak var tagCountLabel: UILabel!
    @IBOutlet weak var deviceSegment: UISegmentedControl!
    
    @IBOutlet weak var stopButton: UIButton!
    @IBOutlet weak var cleearButton: UIButton!
    @IBOutlet weak var readButton: UIButton!
    @IBOutlet weak var batteryImage: UIImageView!
    
    let commonDevice: CommonDevice = CommonDevice.sharedInstance() as! CommonDevice
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.

        CommonDevice.setTriggerModeDefault(true)
        self.commonDevice.setTagCount(0x00, setSacnTime: 0x00, setCycle: 0x00)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.commonDevice.delegateCommon = self
        self.commonDevice.delegateRFID = self
        self.commonDevice.delegateHWEvent = self
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }


/***********************************************************************/
// MARK:Private
/***********************************************************************/
    func pluggedState(){
        DispatchQueue.main.async {
            let info: CommonReaderInfo = CommonReaderInfo.sharedInstance() as! CommonReaderInfo
            
            if(info.bIsPowerOn){
                self.stateLabel.text = "Connected"
            }else{
                self.stateLabel.text = "Plugged"
            }
            
            self.switchOn.isHidden = false
            self.readButton.isEnabled = true
            self.stopButton.isEnabled = true
            
            self.switchOn.setOn(false, animated: false)
            self.title = info.strName
            
            if(info.bCanUseBarcode){
                self.deviceSegment.setEnabled(true, forSegmentAt: 0)
            }else{
                self.deviceSegment.setEnabled(false, forSegmentAt: 0)
            }
            
            
            if(info.bCanUseRFID){
                self.deviceSegment.setEnabled(true, forSegmentAt: 1)
            }else{
                self.deviceSegment.setEnabled(false, forSegmentAt: 1)
            }
            
            if(info.bCanUseNFC){
                self.deviceSegment.setEnabled(true, forSegmentAt: 2)
            }else{
                self.deviceSegment.setEnabled(false, forSegmentAt: 2)
            }
            
        }
    }
    
    func unPluggedState(){
        DispatchQueue.main.async {
            self.disConnectedState()
            self.stateLabel.text = "UnPlugged"
            self.switchOn.setOn(false, animated: true)
            
            
            self.title = "UnknowDevice"
            self.deviceSegment.setEnabled(false, forSegmentAt: 0)
            self.deviceSegment.setEnabled(false, forSegmentAt: 1)
            self.deviceSegment.setEnabled(false, forSegmentAt: 2)
            self.deviceSegment.selectedSegmentIndex = 0
            self.switchOn.isHidden = true
            
            self.readButton.isEnabled = false
            self.stopButton.isEnabled = false
            
            DispatchQueue.main.async {
                self.batteryImage.image = UIImage(named: "bat_normal_0.png")
            }
        }
    }
    
    func connectedState(){
        DispatchQueue.main.async {
            self.stateLabel.text = "Connected"
            
            if !self.switchOn.isOn{
                self.switchOn.setOn(true, animated: false)
            }
        }
    }
    
    func disConnectedState(){
        DispatchQueue.main.async {
            self.stateLabel.text = "DisConnected"
            
            if self.switchOn.isOn{
                self.switchOn.setOn(false, animated: false)
            }
        }
    }
    
    func updateScanData(){
        DispatchQueue.main.async {
            self.tagTableView.reloadData()
            let count: Int = ModelSettings.sharedInstance.readDataList.count
            self.tagCountLabel.text = count.description
        }
    }
    
/***********************************************************************/
// MARK:Delegate
/***********************************************************************/
    open func resPower(onOff isOn: Bool, device nDeviceType: Int32, isHWModeChange bIsHWModeChange: Bool) {
        
        if(isOn)
        {
            /* NFC Initial */
            if( nDeviceType  == RCP_COMBO_DEVICE_NFC)
            {
                let device = ComboNFCApi.sharedInstance() as! ComboNFCApi
                device.setInventory(1.0)
            }
            
        }
        
        if(bIsHWModeChange)
        {
            DispatchQueue.main.async {
                self.deviceSegment.selectedSegmentIndex = Int(nDeviceType)
            }
        }
    }

    func plugged(_ plug: Bool) {
        if(plug){
            NSLog("INFO + plugedState");
            self.pluggedState()
            
        }else{
            NSLog("INFO + unPlugedState");
            self.unPluggedState()

        }
    }
    
    func readerConnected(_ status: UInt8) {
        DispatchQueue.main.async {
            
            switch status{
            case 0xff:
                NSLog("INFO + CONNECTED");
                self.connectedState()
            default:
                NSLog("INFO + DISCONNECTED");
                self.disConnectedState()
                break;
            }
            

        }
    }
    
    func receivedScanData(_ readData: Data!, deviceType nDeviceType: Int32) {
        
        var tag: String = ""
        
        switch nDeviceType{
        case RCP_COMBO_DEVICE_BARCODE:
            tag = String(data: readData, encoding: String.Encoding.shiftJIS)!
            NSLog("['BARCODE : %@", tag)
            /*
            DispatchQueue.main.async {
                let alert = UIAlertController.simpleDialog("test", message: tag)
                self.present(alert, animated: true, completion: nil)
            }
            */
        case RCP_COMBO_DEVICE_RFID:
            tag = readData.hexadecimalString()!
            
        case RCP_COMBO_DEVICE_NFC:
            
            let allData = readData.hexadecimalString()
 
            // NFC DATA FORMAT : STX(1) - ADDR(1) - COMMAND(1) - LEN(1)  - DATA( XX) - ETX(1) - SUM(1) - CR(1)
            let datas = readData.toBytes
            let dataLength = Int(datas[3])
            
            let startIndex = allData?.index((allData?.startIndex)!, offsetBy: 4*2) // 開始位置 2
            let endIndex = allData?.index(startIndex!, offsetBy: dataLength*2) // 長さ 6
            tag = String(allData![startIndex!..<endIndex!])
        default:
            tag =  "Not Found Device Typ"
            break;
        }
        
        var count: Int = 0
        var index: Int = 0
        for dic in ModelSettings.sharedInstance.readDataList {
            if (dic[CellContent.CELL_TAG_INFO] as! String == tag) {
                count = dic[CellContent.CELL_TAG_COUNT] as! Int
                count += 1
                let data = [
                    CellContent.CELL_TAG_INFO : tag,
                    CellContent.CELL_TAG_RAW : readData!,
                    CellContent.CELL_TAG_COUNT: count,
                    CellContent.CELL_TAG_DEVICETYPE: nDeviceType
                ] as [String : Any]
                
                ModelSettings.sharedInstance.readDataList[index] = data
            }
            index += 1
        }
        
        if count == 0 {
            let data = [
                CellContent.CELL_TAG_INFO : tag,
                CellContent.CELL_TAG_RAW : readData!,
                CellContent.CELL_TAG_COUNT: 1,
                CellContent.CELL_TAG_DEVICETYPE: nDeviceType
            ] as [String : Any]
            ModelSettings.sharedInstance.readDataList.append(data)
        }

        self.updateScanData()

    }
    
    func adcReceived(_ data: Data!) {
        
        let b:[UInt8] = data.toBytes
        let adc = b[0]
        var val: Int = Int(adc)
        
        if adc > 95 {
            val = 4
        }else if adc  > 70 {
            val = 3
        }else if adc  > 45 {
            val = 2
        }else if adc  > 20 {
            val = 1
        }else{
            val = 0
        }
        
        NSLog("Battery %d",adc);
        
        let imageName = String(format: "bat_normal_%d.png", val)
        DispatchQueue.main.async {
            self.batteryImage.image = UIImage(named: imageName)
        }
        
    }
    
/***********************************************************************/
// MARK:TableView
/***********************************************************************/
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return ModelSettings.sharedInstance.readDataList.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: "Cell", for: indexPath) as! CustomCell
        let data = ModelSettings.sharedInstance.readDataList[indexPath.row] as NSDictionary
        
        let count: Int = data[CellContent.CELL_TAG_COUNT] as! Int
        cell.tagCount.text = String(count)
        cell.tagHex.text =  data[CellContent.CELL_TAG_INFO] as! String?
        
        let selectedDevice = data[CellContent.CELL_TAG_DEVICETYPE] as! Int32
        
        
        switch selectedDevice{
            case RCP_COMBO_DEVICE_RFID:
                cell.imageView?.image = UIImage(named: "icon_rfid")
            case RCP_COMBO_DEVICE_NFC:
                cell.imageView?.image = UIImage(named: "icon_nfc")
            case RCP_COMBO_DEVICE_BARCODE:
                cell.imageView?.image = UIImage(named: "icon_barcode")
            default:
                break
        }
        

        return cell
    }
    
/***********************************************************************/
// MARK:IBAction
/***********************************************************************/
    
    @IBAction func switchPower(_ sender: Any) {
        
        let sw: UISwitch = sender as! UISwitch
        let selectedType: Int = self.deviceSegment.selectedSegmentIndex
        self.commonDevice.setReaderPower(sw.isOn, buzzer: true, vibration: true, led:true, illumination: true, mode: Int32(selectedType))

    }
    
    @IBAction func secectSegment(_ sender: Any) {
        
        let selectedType: Int = self.deviceSegment.selectedSegmentIndex
        self.commonDevice.setReaderPower(false, buzzer: true, vibration: true, led:true, illumination: true, mode: Int32(selectedType))

    }
    
    @IBAction func tapToRead(_ sender: Any) {
        
        let info: CommonReaderInfo = CommonReaderInfo.sharedInstance() as! CommonReaderInfo
        let tagCount = info.nCount
        let scanTime = info.nScanTime
        let cycle = info.nCycle
        
        let selectedDevice = self.deviceSegment.selectedSegmentIndex
        
        //RFID
        if(selectedDevice == Int(RCP_COMBO_DEVICE_RFID)){
            
            let device: ComboRFIDApi = ComboRFIDApi.sharedInstance() as! ComboRFIDApi
            device.startScan(UInt8(tagCount), mtime: UInt8(scanTime), repeatCycle: UInt16(cycle))
            
        }else if selectedDevice == Int(RCP_COMBO_DEVICE_NFC){
            let device: ComboNFCApi = ComboNFCApi.sharedInstance() as! ComboNFCApi
            device.startScan()
            
        }else if selectedDevice == Int(RCP_COMBO_DEVICE_BARCODE){
            let device: ComboBarcodeApi = ComboBarcodeApi.sharedInstance() as! ComboBarcodeApi
            device.startScan()
        }

    }
    
    @IBAction func tapToClear(_ sender: Any) {
        ModelSettings.sharedInstance.readDataList.removeAll()
        self.updateScanData()
    }
    
    @IBAction func tapToStop(_ sender: Any) {
        
        let selectedDevice = self.deviceSegment.selectedSegmentIndex
        
        //RFID
        if(selectedDevice == Int(RCP_COMBO_DEVICE_RFID)){
            
            let device: ComboRFIDApi = ComboRFIDApi.sharedInstance() as! ComboRFIDApi
            device.stopScan()
            
        }else if selectedDevice == Int(RCP_COMBO_DEVICE_NFC){
            let device: ComboNFCApi = ComboNFCApi.sharedInstance() as! ComboNFCApi
            device.stopScan()
            
        }else if selectedDevice == Int(RCP_COMBO_DEVICE_BARCODE){
            let device: ComboBarcodeApi = ComboBarcodeApi.sharedInstance() as! ComboBarcodeApi
            device.stopScan()
        }

    }
}





